#//-----------------------------------------------
#//TUSB-INS16I用サンプルプログラム
#//Python 用モジュール
#//
#//2024_05_02　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------
import TUSBINSI
import sys

from SpFuncs import *

from functools import partial

DevId = None            # 開いているデバイスのID
mon_t = None            # タイマー管理クラスインスタンス
lock = threading.Lock() # デバイスコマンドが同時実行されないために使用するロック

# Sub Functions -----------------------------

# Widgetの表示を設定する
# stat -> 'Close' , 'Open'
def WidgetState(stat):
    """Widgetの表示を設定する
    
    Arguments:
        stat        ----    設定状態 'Close' or 'Open'
    """
    EnClose = tk.NORMAL if( stat == 'Close' ) else tk.DISABLED
    EnOpen = tk.NORMAL if( stat == 'Open' ) else tk.DISABLED
    IDCmbb['state'] = EnClose
    OpenBtn['state'] = EnClose
    CloseBtn['state'] = EnOpen
    global EdgeDirEn
    EdgeDirEn = ( stat == 'Open' )
    IntervalSpnb['state'] = EnOpen
    if stat != 'Close' :
        form.protocol('WM_DELETE_WINDOW',lambda: 'pass' )
    else :
        form.protocol('WM_DELETE_WINDOW',lambda: form.destroy())

def EdgeSet():
    """エッジ機能を設定する"""
    Data = 0
    for bit in range(16): Data += (1 << bit) if EdgeDirV[bit] else 0 # ディレクション設定配列をビットパターン値に変換する
    Time = IntervalVal.get()
    if lock.acquire(timeout=1):
        ret = TUSBINSI.Tusbins16i_EdgeSet(DevId,Data,Time)
        lock.release()
    if ret != 0:
        ShowInfo(TUSBINSI.GetErrMessage(ret))

# Timer -------------------------------------
def Monitor():
    """入力状態モニタータイマー"""
    global InputSt
    global EdgeSt
    # 入力状態を読み込む
    if lock.acquire(timeout=1):
        ret = TUSBINSI.Tusbins16i_Read(DevId)
        lock.release()
    if ret[0] != 0:return
    for bit in range(16):
        if IsBitSet(ret[1],bit) != IsBitSet(InputSt,bit):
            SetLEDCanvas(InLedCanv[bit], IsBitSet(ret[1],bit))
    InputSt = ret[1]

    # エッジ検出状態を読み込む
    if lock.acquire(timeout=1):
        ret = TUSBINSI.Tusbins16i_EdgeRead(DevId)
        lock.release()
    if ret[0] != 0:return
    for bit in range(16):
        if IsBitSet(ret[1],bit) != IsBitSet(EdgeSt,bit):
            SetLEDCanvas(EdgeLedCanv[bit], IsBitSet(ret[1],bit))
    EdgeSt = ret[1]

# Event Handler -----------------------------

def EdgeDir_on(event,Bit):
    """エッジ検出機能の方向が変更された時に呼び出される
    
    Arguments:
        event           ----    イベント
        Bit             ----    変更されたビット 0~15
    """
    if EdgeDirEn:
        EdgeDirV[Bit] = 0 if EdgeDirV[Bit] == 1 else 1
        SetUDCanvasArrow(EdgeDirCanv[Bit],EdgeDirV[Bit])
        EdgeSet()

def Interval_on():
    """エッジ検出インターバル設定が変更された
    """
    EdgeSet()

def OpenBtn_on():
    """オープンボタン押下"""
    global DevId
    DevId = IDCmbb.current()
    ret = TUSBINSI.Tusbins16i_Device_Open(DevId)
    if ret ==0:
        WidgetState('Open')
        EdgeSet()
        global mon_t
        mon_t = intTT(0.5,Monitor)
        mon_t.Start()
    else:
        ShowInfo(TUSBINSI.GetOpenErrMessage(ret))

def CloseBtn_on():
    """クローズボタン押下"""
    mon_t.Stop()
    TUSBINSI.Tusbins16i_Device_Close(DevId)
    WidgetState('Close')

# form generation ---------------------------
form  = tk.Tk()
form.geometry('330x501')
form.title('TUSB-INS16I Sample Program')
form.attributes("-toolwindow",True)

# 見出しラベル
GetLabel(form,"Bit",0.15,0.05)
GetLabel(form,"Input",0.28,0.05)
GetLabel(form,"Edge",0.48,0.05)

# ビット毎表示項目
InLedCanv =list()
InputSt = 0
EdgeDirCanv = list()
EdgeDirV = [1 for i in range(16)]
EdgeDirEn = False
EdgeLedCanv = list()
EdgeSt = 0
bypit = 0.044    # 行ピッチ
for bit in range(16):
    ly = 0.1+bypit*bit
    GetLabel(form,str(bit),0.15,ly)
    InLedCanv.append(GetLEDCanvas(form,0.31,ly))
    SetLEDCanvas(InLedCanv[bit],False)
    EdgeDirCanv.append(GetUDCanvas(form,0.53,ly,partial(EdgeDir_on,Bit=bit)))
    SetUDCanvasArrow(EdgeDirCanv[bit],EdgeDirV[bit])
    EdgeLedCanv.append(GetLEDCanvas(form,0.60,ly))
    SetLEDCanvas(EdgeLedCanv[bit],False)

# 凡例
LegendFrame = GetFrame(form,0.3,0.2,0.68,0.35)
SetUDCanvasArrow(GetUDCanvas(LegendFrame,0.15,0.15,None),0)
GetLabel(LegendFrame,"Off->On Edge",0.0,0.4)
SetUDCanvasArrow(GetUDCanvas(LegendFrame,0.15,0.65,None),1)
GetLabel(LegendFrame,"On->Off Edge",0.0,0.9)

# インターバル
IntervalFrame = GetLFrame(form,"Interval",0.3,0.2,0.68,0.6)
GetLabel(IntervalFrame,"t=",0.1,0.1)
IntervalVal = tk.IntVar(value=0)
IntervalSpnb = GetSpinBoxN(IntervalFrame,0,255,1,IntervalVal,0.8,0.1,0.4,Interval_on)
GetLabel(IntervalFrame,"t+1[ms]",0.2,0.8)

#ID open close
IDList = [str(i) for i in range(16)]
GetLabel(form,"ID",0.05,0.85)
IDCmbb = GetComboBox(form,IDList,0.15,0.15,0.85)
OpenBtn = GetButton(form,"DevOpen",OpenBtn_on,0.25,0.17,0.93)
CloseBtn = GetButton(form,"DevClose",CloseBtn_on,0.25,0.5,0.93)

# Start
WidgetState('Close')
form.mainloop()
